==> 0ad_0.27.1.orig.tar.xz: mozjs-115.16.1/js/src/jit-test/tests/debug/Memory-takeCensus-06.js <==
// Check Debugger.Memory.prototype.takeCensus handling of 'breakdown' argument.

load(libdir + 'match.js');
var Pattern = Match.Pattern;

var g = newGlobal({newCompartment: true});
var dbg = new Debugger(g);

Pattern({ count: Pattern.NATURAL,
          bytes: Pattern.NATURAL })
  .assert(dbg.memory.takeCensus({ breakdown: { by: 'count' } }));

let census = dbg.memory.takeCensus({ breakdown: { by: 'count', count: false, bytes: false } });
assertEq('count' in census, false);
assertEq('bytes' in census, false);

census = dbg.memory.takeCensus({ breakdown: { by: 'count', count: true,  bytes: false } });
assertEq('count' in census, true);
assertEq('bytes' in census, false);

census = dbg.memory.takeCensus({ breakdown: { by: 'count', count: false, bytes: true } });
assertEq('count' in census, false);
assertEq('bytes' in census, true);

census = dbg.memory.takeCensus({ breakdown: { by: 'count', count: true,  bytes: true } });
assertEq('count' in census, true);
assertEq('bytes' in census, true);


// Pattern doesn't mind objects with extra properties, so we'll restrict this
// list to the object classes we're pretty sure are going to stick around for
// the forseeable future.
Pattern({
          Function:          { count: Pattern.NATURAL },
          Object:            { count: Pattern.NATURAL },
          DebuggerPrototype: { count: Pattern.NATURAL },
          global:            { count: Pattern.NATURAL },
        })
  .assert(dbg.memory.takeCensus({ breakdown: { by: 'objectClass' } }));

Pattern({
          objects:        { count: Pattern.NATURAL },
          scripts:        { count: Pattern.NATURAL },
          strings:        { count: Pattern.NATURAL },
          other:          { count: Pattern.NATURAL }
        })
  .assert(dbg.memory.takeCensus({ breakdown: { by: 'coarseType' } }));

// As for { by: 'objectClass' }, restrict our pattern to the types
// we predict will stick around for a long time.
Pattern({
          JSString:             { count: Pattern.NATURAL },
          'js::Shape':          { count: Pattern.NATURAL },
          JSObject:             { count: Pattern.NATURAL },
        })
  .assert(dbg.memory.takeCensus({ breakdown: { by: 'internalType' } }));


// Nested breakdowns.

let coarse_type_pattern = {
  objects:        { count: Pattern.NATURAL },
  scripts:        { count: Pattern.NATURAL },
  strings:        { count: Pattern.NATURAL },
  other:          { count: Pattern.NATURAL }
};

Pattern({
          JSString:    coarse_type_pattern,
          'js::Shape': coarse_type_pattern,
          JSObject:    coarse_type_pattern,
        })
  .assert(dbg.memory.takeCensus({
    breakdown: { by: 'internalType',
                 then: { by: 'coarseType' }
    }
  }));

Pattern({
          Function:          { count: Pattern.NATURAL },
          Object:            { count: Pattern.NATURAL },
          DebuggerPrototype: { count: Pattern.NATURAL },
          global:            { count: Pattern.NATURAL },
          other:             coarse_type_pattern
        })
  .assert(dbg.memory.takeCensus({
    breakdown: {
      by: 'objectClass',
      then:  { by: 'count' },
      other: { by: 'coarseType' }
    }
  }));

Pattern({
          objects: { count: Pattern.NATURAL, label: "object" },
          scripts: { count: Pattern.NATURAL, label: "scripts" },
          strings: { count: Pattern.NATURAL, label: "strings" },
          other:   { count: Pattern.NATURAL, label: "other" }
        })
  .assert(dbg.memory.takeCensus({
    breakdown: {
      by: 'coarseType',
      objects: { by: 'count', label: 'object' },
      scripts: { by: 'count', label: 'scripts' },
      strings: { by: 'count', label: 'strings' },
      other:   { by: 'count', label: 'other' }
    }
  }));

==> 0ad_0.28.0.orig.tar.xz: mozjs-128.13.0/js/src/jit-test/tests/debug/Memory-takeCensus-06.js <==
// Check Debugger.Memory.prototype.takeCensus handling of 'breakdown' argument.

load(libdir + 'match.js');
var Pattern = Match.Pattern;

var g = newGlobal({newCompartment: true});
var dbg = new Debugger(g);

Pattern({ count: Pattern.NATURAL,
          bytes: Pattern.NATURAL })
  .assert(dbg.memory.takeCensus({ breakdown: { by: 'count' } }));

let census = dbg.memory.takeCensus({ breakdown: { by: 'count', count: false, bytes: false } });
assertEq('count' in census, false);
assertEq('bytes' in census, false);

census = dbg.memory.takeCensus({ breakdown: { by: 'count', count: true,  bytes: false } });
assertEq('count' in census, true);
assertEq('bytes' in census, false);

census = dbg.memory.takeCensus({ breakdown: { by: 'count', count: false, bytes: true } });
assertEq('count' in census, false);
assertEq('bytes' in census, true);

census = dbg.memory.takeCensus({ breakdown: { by: 'count', count: true,  bytes: true } });
assertEq('count' in census, true);
assertEq('bytes' in census, true);


// Pattern doesn't mind objects with extra properties, so we'll restrict this
// list to the object classes we're pretty sure are going to stick around for
// the forseeable future.
Pattern({
          Function:          { count: Pattern.NATURAL },
          Object:            { count: Pattern.NATURAL },
          DebuggerPrototype: { count: Pattern.NATURAL },
          global:            { count: Pattern.NATURAL },
        })
  .assert(dbg.memory.takeCensus({ breakdown: { by: 'objectClass' } }));

Pattern({
          objects:        { count: Pattern.NATURAL },
          scripts:        { count: Pattern.NATURAL },
          strings:        { count: Pattern.NATURAL },
          other:          { count: Pattern.NATURAL }
        })
  .assert(dbg.memory.takeCensus({ breakdown: { by: 'coarseType' } }));

// As for { by: 'objectClass' }, restrict our pattern to the types
// we predict will stick around for a long time.
Pattern({
          JSString:             { count: Pattern.NATURAL },
          'js::Shape':          { count: Pattern.NATURAL },
          JSObject:             { count: Pattern.NATURAL },
        })
  .assert(dbg.memory.takeCensus({ breakdown: { by: 'internalType' } }));


// Nested breakdowns.

let coarse_type_pattern = {
  objects:        { count: Pattern.NATURAL },
  scripts:        { count: Pattern.NATURAL },
  strings:        { count: Pattern.NATURAL },
  other:          { count: Pattern.NATURAL }
};

Pattern({
          JSString:    coarse_type_pattern,
          'js::Shape': coarse_type_pattern,
          JSObject:    coarse_type_pattern,
        })
  .assert(dbg.memory.takeCensus({
    breakdown: { by: 'internalType',
                 then: { by: 'coarseType' }
    }
  }));

Pattern({
          Function:          { count: Pattern.NATURAL },
          Object:            { count: Pattern.NATURAL },
          DebuggerPrototype: { count: Pattern.NATURAL },
          global:            { count: Pattern.NATURAL },
          other:             coarse_type_pattern
        })
  .assert(dbg.memory.takeCensus({
    breakdown: {
      by: 'objectClass',
      then:  { by: 'count' },
      other: { by: 'coarseType' }
    }
  }));

Pattern({
          objects: { count: Pattern.NATURAL, label: "object" },
          scripts: { count: Pattern.NATURAL, label: "scripts" },
          strings: { count: Pattern.NATURAL, label: "strings" },
          other:   { count: Pattern.NATURAL, label: "other" }
        })
  .assert(dbg.memory.takeCensus({
    breakdown: {
      by: 'coarseType',
      objects: { by: 'count', label: 'object' },
      scripts: { by: 'count', label: 'scripts' },
      strings: { by: 'count', label: 'strings' },
      other:   { by: 'count', label: 'other' }
    }
  }));

try {
  const breakdown = { by: "objectClass" };
  breakdown.then = breakdown;
  dbg.memory.takeCensus({ breakdown });
  assertEq(true, false, "should not reach here");
} catch (e) {
  assertEq(e.message, "takeCensus breakdown 'by' value nested within itself: \"objectClass\"");
}

try {
  const breakdown = { by: "objectClass", then: { by: "objectClass" } };
  dbg.memory.takeCensus({ breakdown });
  assertEq(true, false, "should not reach here");
} catch (e) {
  assertEq(e.message, "takeCensus breakdown 'by' value nested within itself: \"objectClass\"");
}

try {
  const breakdown = { by: "coarseType", scripts: { by: "filename" } };
  breakdown.scripts.noFilename = breakdown;
  dbg.memory.takeCensus({ breakdown });
  assertEq(true, false, "should not reach here");
} catch (e) {
  assertEq(e.message, "takeCensus breakdown 'by' value nested within itself: \"coarseType\"");
}
